#!/usr/bin/python

ANSIBLE_METADATA = {
    "metadata_version": "1.0",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: add_header_rule

short_description: Module to add or modify an addHeaderRules on a LoadMaster.

version_added: "1.0"

description:
    - "Module to add or modify an addHeaderRules on a LoadMaster. The minumum
    supported LoadMaster firmware virsion is 7.2.47.0. Rules must be defined in
    your playbook before being assigned to Virtual Services, Sub Virtual 
    Services and Real Servers."

options:
    lm_address:
        description:
            - IP address and port of the LoadMaster. The format is 'ip:port'.
        required: true
        type: str
    central_address:
        description:
            - The Central IP address where the LoadMaster is added to.
        required: true
        type: str
    username:
        description:
            - The Central username of the user.
        required: true
        type: str
    api_key:
        description:
            - The API key for the user of the Central machine.
        required: true
        type: str
    name:
        description:
            - The name of the AddHeaderRule
        required: true
        type: str
    header:
        description:
            - Name of the header field to be added.
        required: true
        type: str
    replacement:
        description:
            - The replacement string. You can enter a maximum of 255 characters 
            in this parameter.
        required: true
        type: str
    only_on_flag:
        description:
            - Range: 1-9. Only try to execute this rule if the specified flag 
            is set. Using the only_on_flag and setonmatch paramters, it is 
            possible to make rules dependent on each other, that is, only
            execute a particular rule if another rule has been successfully 
            matched
        required: false
        type: int
"""

EXAMPLES = """
# Add Header Rules
- name: Create AddHeaderRule
  hosts: localhost

  vars:
    central_address: '10.35.39.21'
    lm_address: '10.35.39.20:443'
    username: 'admin'
    api_key: '699129a26ace3fcd34466a4cc'

  tasks:
    - name: Create AddHeaderRule
      add_header_rule:
        lm_address: '{{ lm_address }}'
        central_address: '{{ central_address }}'
        username: '{{ username }}'
        api_key: '{{ api_key }}'
        name: 'addHeaderRule1'
        header: 'name'
        replacement: 'username'
"""

RETURN = """
message:
    description: The message response indicating whether the task created or modified
    the rule
    type: str
    returned: always
    sample: AddHeaderRule with name addHeaderRule1 was created successfully
changed:
    description: A boolean to indicate whether changes were made during the task
    type: bool
    returned: always
    sample: true
dataChanged:
    description: The parameters that were changed during the task
    type: str
    returned: when changed is true
    sample: {"Header": "name","HeaderValue": "username","Name": "addHeaderRule1"}
msg:
    description: The error message related to why the task failed.
    type: str
    returned: when task failed
    sample:  The minimum supported LoadMaster firmware version is 7.2.47.0
"""

from ansible.module_utils.basic import AnsibleModule
import requests
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

API_MAP = {
    "central_address": "central_address",
    "header": "header",
    "lm_address": "lm_address",
    "name": "name",
    "onlyonflag": "only_on_flag",
    "replacement": "replacement",
}


def run_module():
    module_args = dict(
        api_key=dict(type="str", required=True),
        central_address=dict(type="str", required=True),
        header=dict(type="str", required=True),
        lm_address=dict(type="str", required=True),
        name=dict(type="str", required=True),
        only_on_flag=dict(
            type="int",
            choices=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"],
        ),
        replacement=dict(type="str", required=True),
        username=dict(type="str", required=True),
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)

    cred = {
        "username": module.params.pop("username"),
        "api_key": module.params.pop("api_key"),
    }

    add_header_rule = map_params_to_api(module.params)
    add_header_rule["type"] = "1"

    central_access_uri = "https://{}/api/v1/cms/contentrules/".format(
        module.params["central_address"]
    )

    response = requests.post(
        central_access_uri, json=add_header_rule, params=cred, verify=False
    )

    resp = response.json()

    if response.status_code is not 200:
        result = dict(changed=False)
        module.fail_json(msg=resp["message"], **result)

    if "dataChanged" in resp:
        result = dict(changed=False, dataChanged="", message="")
        result["message"] = resp["message"]
        result["dataChanged"] = resp["dataChanged"]
        result["changed"] = resp["changed"]
    else:
        result = dict(changed=False, message="")
        result["message"] = resp["message"]
        result["changed"] = resp["changed"]

    if module.check_mode:
        module.exit_json(**result)

    module.exit_json(**result)


def map_params_to_api(params):
    """Map the values from the playbook to the Values for the API."""
    transformed = {}
    for api_key, input_key in API_MAP.items():
        if params.get(input_key, None):
            transformed[api_key] = params[input_key]
    return transformed


def main():
    run_module()


if __name__ == "__main__":
    main()
