#!/usr/bin/python

ANSIBLE_METADATA = {
    "metadata_version": "1.0",
    "status": ["preview"],
    "supported_by": "communty",
}

DOCUMENTATION = """
---
module: cert_management

short_description: Module to upload certificate and key on a LoadMaster.

version_added: "1.0"

description:
    - Module to upload certificate and key to a LoadMaster. A certificate and key 
    must be in the same file being uploaded. A certificate upload must be defined
    in your playbook before being assigned to a Virtual Service.

options:
    api_key:
        description:
            -   The API key for the user of the Central machine.
        required: true
        type: str
    central_address:
        description:
            -   The Central IP address where the LoadMaster is added to.
        required: true
        type: str
    cert_name:
        description:
            -   Name of the identifier of the cert to upload or replace.
        required: true
        type: str
    cert_file:
        description:
            -   Path to the file where the key and cert is stored
                This must have both key and cert in the same file.
        required: true
        type: str
    replace:
        description:
            - A boolean to upload the cert to replace the current cert.
        required: true
        type: int
        choices:
            - 0
            - 1
    username:
        description:
            -   The Central username of the user.
        required: true
        type: int
    intermediate:
        description:
            -   A boolean to specify the cert is an intermediate or not
        type: int
        choices:
            - 0
            - 1
        default: 0
"""

EXAMPLES = """
- name: Upload a certificate to the LoadMaster
  hosts: localhost

  vars:
    central_address: '10.35.39.21'
    lm_address: '10.35.39.20:443'
    username: 'admin'
    api_key: '699129a26ace3fcd34466a4cc'

  tasks:
  - name: Upload a certificate to the LoadMaster
    cert_management:
      lm_address: '{{ lm_address }}'
      central_address: '{{ central_address }}'
      cert_name: 'cert'
      cert_file: '/path/to/cert/test.pem'
      replace: 0
      username: '{{ username }}'
      api_key: '{{ api_key }}'
"""

RETURN = '''
message:
    description: The message response indicating whether the cert was uploaded
    type: str
    returned: always
    sample: Certificate uploaded to LoadMaster
changed:
    description: A boolean to indicate whether changes were made during the task
    type: bool
    returned: always
    sample: true
msg:
    description: The error message related to why the task failed.
    type: str
    returned: when task failed
    sample:  Could not add Certificate to  LM - Command Failed: Certificate Identifier already exists
'''

from ansible.module_utils.basic import AnsibleModule
import requests
import urllib3
urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

API_MAP = {
    "central_address": "central_address",
    "cert_name": "cert_name",
    "intermediate": "intermediate",
    "lm_address": "lm_address",
    "password": "password",
    "replace": "replace",
}


def check_if_file_exists(path):
    import os
    return os.path.exists(path)


def run_module():
    spec = dict(
        central_api_key=dict(required=True),
        central_address=dict(required=True),
        central_username=dict(),
        cert_file=dict(type="path", required=True),
        cert_name=dict(required=True),
        intermediate=dict(type="int", default=0, choices=[0,1]),
        lm_address=dict(required=True),
        password=dict(),
        replace=dict(type="int", required=True, choices=[0,1])
    )

    module = AnsibleModule(argument_spec=spec, supports_check_mode=True)
    result = dict(changed=False, original_message="", message="")

    if module.check_mode:
        module.exit_json(**result)

    central_auth = dict(
        username=module.params.pop("central_username"),
        api_key=module.params.pop("central_api_key")
    )

    result = dict(changed=False)
    resp = {}
    if check_if_file_exists(module.params['cert_file']):

        file = open(module.params.pop('cert_file'), 'rb')

        central_access_uri = "https://{}/api/v1/cms/cert/".format(
            module.params.pop("central_address")
        )

        response = requests.post(central_access_uri,
                                 data=map_params_to_api(module.params),
                                 files={'cert_file': file},
                                 params=central_auth,
                                 verify=False)
        resp = response.json()
    else:
        resp['message'] = 'Cert path does not exist'
        module.fail_json(msg=resp['message'], **result)

    if response.status_code is not 200:
        module.fail_json(msg=resp['message'], **result)

    result['changed'] = True
    result['message'] = resp['message']
    module.exit_json(**result)



def map_params_to_api(params):
    """Map the values from the playbook to the Values for the API."""
    transformed = {}
    for api_key, input_key in API_MAP.items():
        if params.get(input_key, None):
            transformed[api_key] = params[input_key]
    return transformed


def main():
    run_module()


if __name__ == "__main__":
    main()
