#!/usr/bin/python

ANSIBLE_METADATA = {
    "metadata_version": "1.0",
    "status": ["preview"],
    "supported_by": "communty",
}

DOCUMENTATION = """
---
module: real_server

short_description: Module to add or modify a Real Server on a LoadMaster.

version_added: "1.0"

description:
    - "Module to add or modify a real server to Virtual Services and
    Sub Virtual Services on a LoadMaster. The minumum supported LoadMaster 
    firmware virsion is 7.2.47.0. To configure a Real Server on a Virtual 
    Service, the Virtual Service must be defined in your playbook before the 
    Real Server. To configure a Real Server on a Sub Virtual Service, the Sub 
    Virtual Service must be defined in your playbook before the Real Server."

options:
    lm_address:
        description:
            - IP address and port of the LoadMaster that contains the Virtual
            Service or Sub Virtual Service that the Real Server should be
            created on modified on. The format is 'ip:port'.
        required: true
        type: str
    central_address:
        description:
            - The Central IP address where the LoadMaster is added to.
        required: true
        type: str
    username:
        description:
            - The Central username of the user.
        required: true
        type: str
    api_key:
        description:
            - The API key for the user of the Central machine.
        required: true
        type: str
    vs_ip:
        description:
            - The IP address of the Virtual Service on the provided LoadMaster.
        required: true
        type: str
    vs_port:
        description:
            - The port of the Virtual Service on the provided LoadMaster. 
            Values are between 3 and 65530.
        required: true
        type: int
    vs_prot:
        description:
            - The protocol of the Virtual Service on the provided LoadMaster.
        required: true
        choices: 
            - udp
            - tcp
        type: str
    rs_ip:
        description:
            - The IP address of the Real Server that is being created or 
            modified
        required: true
        type: str
    rs_port: 
        description:
            - The port of the Real Server that is being created or modified. 
            Values are between 3 and 65530.
        required: true
        type: int
    rs_limit:
        description:
            - The maximum number of open connections that can be sent to a Real
            Server before it is taken out of rotation. Values are between 0 and
            100000.
        required: false
        type: int
    rs_weight:
        description:
            - When using weighted round robin scheduling, the weight of a Real 
            Server is used to indicate what relative proportion of traffic
            should be sent to the server. Servers with higher values will
            receive more traffic.
            - The weight of a SubVS can also be updated using the modrs
            command, set the rs to the number that appears in the Id column for
            the relevant SubVS in the parent Virtual Service modify screen.
        required: false
        type: int
    rs_fw_method:
        description:
            - The type of forwarding method used. The default method is NAT. 
            Direct server return can only be used with Layer 4 services.
        required: false
        choices: nat and route
        type: str
    rs_enable:
        description:
            - Enable or disable the Real Server.
        required: false
        choices: 
            - Y
            - N
        default: Y
        type: str
    rs_critical:
        description:
            - Enabling this parameter indicates that the Real Server is
            required for the Virtual Service to be considered available. The
            Virtual Service will be marked as down if the Real Server has
            failed or is disabled.
        required: false
        choices: 
            - 0: Disabled
            - 1: Enabled
        type: int
    sub_vs_nickname:
        description:
            - To create or modify a Real Server on a Sub Virtual Service, the
            nickname of the Sub Virtual Service must be provided.
        required: false
        type: str
    addtoallsubvs:
        description:
            - Enable this option when adding a Real Server to all SubVSs of a 
            Virtual Service. Values are 0 or 1.
        required: false
        choices: 
            - 0: Disabled
            - 1: Enabled
        type: int
    newport:
        description:
            - The port on the Real Server to be used. Values are between 3 and
            65535.
        required: false
        type: int
    follow:
        description:
            - Specify what Real Server the health check is based on by setting
            this parameter to the RsIndex of the Real Server to be followed. 
            This can either be set to the RsIndex of the same Real Server to 
            health check based on that particular Real Server status, or 
            another Real Server can be specified. For example - if Real Server
            1 is down, any Real Servers which have their health check based 
            on Real Server 1 will also be marked as down, regardless of their
            actual Real Server status.
        required: false
        type: int
    content_rules:
        description:
            - A list of content rule names to added to a Real Server. The names
            provided must be previously added to the LoadMaster, and must be
            Content Matching rules.
        required: false
        type: list

"""

EXAMPLES = """
# Configure Real Servers
- name: Create Real Server
  hosts: localhost

  vars:
    central_address: '10.35.39.21'
    lm_address: '10.35.39.20:443'
    username: 'admin'
    api_key: '699129a26ad34466a4cc'
  
  tasks:
- name: Create Real Server
  hosts: localhost
  tasks:
  - name: Create RS for VS 10.35.39.25:8010
    real_server:
      lm_address: '{{ lm_address }}'
      central_address: '{{ central_address }}'
      username: '{{ username }}'
      api_key: '{{ api_key }}'
      vs_ip: '10.35.39.25'
      vs_port: 8010
      vs_prot: 'tcp'
      rs_ip: '10.35.39.6'
      rs_port: 4006
      rs_limit: 220
"""

RETURN = '''
message:
    description: The message response indicating whether the task created or modified
    the Real Server
    type: str
    returned: always
    sample: Real Server 10.35.39.180:8010 created successfully
changed:
    description: A boolean to indicate whether changes were made during the task
    type: bool
    returned: always
    sample: true
dataChanged:
    description: The parameters that were changed during the task
    type: str
    returned: when changed is true
    sample: {"Addr": "10.35.39.180", "Critical": "N", "DnsName": null, "Enable": "Y", "Follow": "0", "Forward": "nat"}
msg:
    description: The error message related to why the task failed.
    type: str
    returned: when task failed
    sample:  The minimum supported LoadMaster firmare version is 7.2.47.0
'''

from ansible.module_utils.basic import AnsibleModule
import requests
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

API_MAP = {
    "lm_address": "lm_address",
    "central_address": "central_address",
    "vs": "vs_ip",
    "port": "vs_port",
    "prot": "vs_prot",
    "rs": "rs_ip",
    "rsport": "rs_port",
    "Limit": "rs_limit",
    "Weight": "rs_weight",
    "Forward": "rs_fw_method",
    "Enable": "rs_enable",
    "Critical": "rs_critical",
    "sub_vs_nickname": "sub_vs_nickname",
    "addtoallsubvs": "addtoallsubvs",
    "newport": "newport",
    "follow": "follow",
    "ContentRules": "content_rules",
}


def run_module():
    module_args = dict(
        addtoallsubvs=dict(type="int", choices=[0,1]),
        api_key=dict(type="str", required=True),
        central_address=dict(type="str", required=True),
        content_rules=dict(type="list"),
        follow=dict(type="int"),
        lm_address=dict(type="str", required=True),
        newport=dict(type="int"),
        rs_critical=dict(type="int", choices=[0,1]),
        rs_enable=dict(type="str", default="Y", choices=["Y","N"]),
        rs_fw_method=dict(type="str", choices=["nat", "route"]),
        rs_ip=dict(type="str", required=True),
        rs_limit=dict(type="int"),
        rs_port=dict(type="int", required=True),
        rs_weight=dict(type="int"),
        sub_vs_nickname=dict(type="str"),
        username=dict(type="str", required=True),
        vs_ip=dict(type="str", required=True),
        vs_port=dict(type="int", required=True),
        vs_prot=dict(type="str", required=True, choices=["udp", "tcp"]),
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)

    central_auth = dict(
        username=module.params.pop("username"), api_key=module.params.pop("api_key")
    )

    vs_info = dict(
        vs=module.params.pop("vs_ip"),
        port=module.params.pop("vs_port"),
        prot=module.params.pop("vs_prot"),
    )
    
    real_server = map_params_to_api(module.params)

    params = dict(vs_info=vs_info, real_server=real_server)

    central_access_uri = "https://{}/api/v1/cms/rs/".format(
        module.params["central_address"]
    )

    response = requests.post(
        central_access_uri, params=central_auth, json=params, verify=False
    )

    resp = response.json()

    if response.status_code is not 200:
        result = dict(changed=False)
        module.fail_json(msg=resp["message"], **result)

    if "dataChanged" in resp:
        result = dict(changed=False, dataChanged="", message="")
        result["message"] = resp["message"]
        result["dataChanged"] = resp["dataChanged"]
        result["changed"] = resp["changed"]
    else:
        result = dict(changed=False, message="")
        result["message"] = resp["message"]
        result["changed"] = resp["changed"]

    if module.check_mode:
        module.exit_json(**result)

    module.exit_json(**result)


def map_params_to_api(params):
    """Map the values from the playbook to the Values for the API."""
    transformed = {}
    for api_key, input_key in API_MAP.items():
        if params.get(input_key, None):
            transformed[api_key] = params[input_key]
    return transformed


def main():
    run_module()


if __name__ == "__main__":
    main()
