#!/usr/bin/python

ANSIBLE_METADATA = {
    "metadata_version": "1.0",
    "status": ["preview"],
    "supported_by": "community",
}

DOCUMENTATION = """
---
module: replace_header_rule

short_description: Module to add or modify a replaceHeaderRule on a LoadMaster.

version_added: "1.0"

description:
    - "Module to add or modify a replaceHeaderRule to a LoadMaster. The minumum
    supported LoadMaster firmware virsion is 7.2.47.0. Rules must be defined in
    your playbook before being assigned to Virtual Services, Sub Virtual 
    Services and Real Servers."

options:
    lm_address:
        description:
            - IP address and port of the LoadMaster. The format is 'ip:port'.
        required: true
        type: str
    central_address:
        description:
            - The Central IP address where the LoadMaster is added to.
        required: true
        type: str
    username:
        description:
            - The Central username of the user.
        required: true
        type: str
    api_key:
        description:
            - The API key for the user of the Central machine.
        required: true
        type: str
    name:
        description:
            - The name of the ReplaceHeaderRule
        required: true
        type: str
    header:
        description:
            - The header field name where the substitution should be performed.
        required: true
        type: str
    replacement:
        description:
            - The replacement string. 
        required: true
        type: str
    pattern:
        description:
            - The pattern to be matched.
        required: false
        type: str
    only_on_flag:
        description:
            - Range: 1-9. Only try to execute this rule if the specified flag 
            is set. Using the onlyonflag and setonmatch paramters, it is 
            possible to make rules dependent on each other, that is, only
            execute a particular rule if another rule has been successfully 
            matched
        required: false
        type: int

"""

EXAMPLES = """
# Replace Header Rule
- name: Create ReplaceHeaderRule
  hosts: localhost

  vars:
    central_address: '10.35.39.21'
    lm_address: '10.35.39.20:443'
    username: 'admin'
    api_key: '699129a26ace406fd65ee30a6983fcd34466a4cc'

  tasks:
  - name: Create ReplaceHeaderRule
    replace_header_rule:
        lm_address: '{{ lm_address }}'
        central_address: '{{ central_address }}'
        username: '{{ username }}'
        api_key: '{{ api_key }}'
        name: 'replaceHeaderRule1'
        header: 'name'
        replacement: 'username'
        pattern: '^((http[s]?|ftl):\/)$'
"""

RETURN = '''
message:
    description: The message response indicating whether the task created or modified
    the rule
    type: str
    returned: always
    sample: ReplaceHeaderRule with name replaceHeaderRule1 was created successfully
changed:
    description: A boolean to indicate whether changes were made during the task
    type: bool
    returned: always
    sample: true
dataChanged:
    description: The parameters that were changed during the task
    type: str
    returned: when changed is true
    sample: {"Header": "name","Name": "replaceHeaderRule1", "Pattern": "^((http[s]?|ftl):\\/)$",Replacement": "username"}
msg:
    description: The error message related to why the task failed.
    type: str
    returned: when task failed
    sample:  The minimum supported LoadMaster firmware version is 7.2.47.0
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils import utils
import requests
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

API_MAP = {
    "central_address": "central_address",
    "header": "header",
    "lm_address": "lm_address",
    "name": "name",
    "onlyonflag": "only_on_flag",
    "pattern": "pattern",
    "replacement": "replacement",
    "username": "username",
}


def run_module():
    module_args = dict(
        api_key=dict(type="str", required=True),
        central_address=dict(type="str", required=True),
        header=dict(type="str"),
        lm_address=dict(type="str", required=True),
        name=dict(type="str", required=True),
        only_on_flag=dict(
            type="int",
            choices=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"],
        ),
        pattern=dict(type="str"),
        replacement=dict(type="str"),
        username=dict(type="str", required=True),
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)

    cred = {
        'username': module.params.pop('username'),
        'api_key': module.params.pop('api_key')
    }

    replace_header_rule = utils.map_params_to_api(module.params, API_MAP)
    replace_header_rule["type"] = "3"

    central_address = replace_header_rule.pop("central_address")
    central_access_uri = "https://{}/api/v1/cms/contentrules/".format(
        central_address
    )
    response = requests.post(
        central_access_uri, json=replace_header_rule, params=cred, verify=False
    )

    resp = response.json()
    if response.status_code is not 200:
        result = dict(changed=False)
        module.fail_json(msg=resp["message"], **result)

    if "dataChanged" in resp:
        result = dict(changed=False, dataChanged="", message="")
        result["message"] = resp["message"]
        result["dataChanged"] = resp["dataChanged"]
        result["changed"] = resp["changed"]
    else:
        result = dict(changed=False, message="")
        result["message"] = resp["message"]
        result["changed"] = resp["changed"]

    if module.check_mode:
        module.exit_json(**result)

    module.exit_json(**result)


def main():
    run_module()


if __name__ == "__main__":
    main()
