#!/usr/bin/python

ANSIBLE_METADATA = {
    "metadata_version": "1.0",
    "status": ["preview"],
    "supported_by": "kemp",
}

DOCUMENTATION = """
---
module: sub_virtual_service

short_description: Module to add or modify a Sub Virtual Service on a LoadMaster.

version_added: '1.0'

description:
    - 'Module to add or modify a Sub Virtual Service on a Virtual Service. The 
    minumum supported LoadMaster firmware virsion is 7.2.47.0. To configure a
    Sub Virtual Service on a Virtual Service, the Virtual Service must be 
    defined in your playbook before the Sub Virtual Service.'

options:
    add_via:
        description:
            -   Corresponds to the add http headers in LM
        choices:
            - 0 
            - 1
            - 2
            - 3
            - 4
            - 5
            - 6
        required: false
        type: int
    central_username:
        description:
            -   The Central username of the user.
        required: true
        type: str
    central_api_key:
        description:
            -   The API key for the user of the Central machine.
        required: true
        type: str
     central_address:
        description:
            -   The Central IP address where the LoadMaster is added to.
        required: true
        type: str
    enable:
        description:
            -   Enable the sub virtual service
        choices:
              - Y
              - N
        type: str 
    lm_address:
        description:
            -   IP address and port of the LoadMaster that contains the Virtual
                Service or Sub Virtual Service that the Real Server should be
                created on modified on. The format is 'ip:port'.
        required: true
        type: str
    vs:
        description:
            -   The IP address of the parent Virtual Service on the LoadMaster.
        required: true
        type: str
    port:
        description:
            -   The port of the parent Virtual Service on the LoadMaster Value
            between 3-65530.
        required: true
        type: int
    limit:
        description:
            -	The maximum number of open connections that can be sent to a 
            Real Server before it is taken out of rotation. values 0-100000.
        required: false
        type: int
    nickname:
        description:
            - Nickname of a sub virtual service.
        required: true
        type: str
    qos:
        description:
            -   Quality of Service sets a type of service that deals with 
                packets will treat and prioritise the traffic
        required: false
        type: int
        choices:
            - 0
            - 1
            - 2
            - 4
            - 8
    subnet_originating:
        description:
            -   When transparency is not enabled, the source IP address of 
                connections to the Real Servers is that of the Virtual Service. 
                When transparency is enabled, the source IP address will be the 
                IP address that is initiating connection to the 
                Virtual Service. If the Real Server is on a subnet, and the 
                Subnet Originating Requests option is enabled, 
                then the subnet address of the LoadMaster will be used as 
                the source IP address.
        required: false
        type: int
        choices: 
            - 0
            - 1
    vs_type:
        description:
            - Specifies the type of service being load balanced
        type: str
        choices: 
            - gen
            - http
            - http2
            - tls
            - log
    critical:
        description:
            -   Enabling this parameter indicates that the Real Server 
                is required for the Virtual Service to be considered available. 
                The Virtual Service will be marked as down if the Real Server 
                has failed or is disabled.
        type: int
        choices:
            - 0
            - 1
    check_type:
        description:
            -   Specify which protocol is to be used to check the health 
                of the Real Server.
        type: str
        choices: 
            - icmp
            - http 
            - https 
            - tcp 
            - smtp 
            - nntp 
            - ftp 
            - telnet 
            - pop3
            - imap 
            - rdp 
            - bdara 
            - ldap 
            - none
    check_codes:
        description:
            -   A space-separated list of HTTP status codes that should be 
                treated as successful when received from the Real Server.
        type: str
    check_port:
        description:
            -   The port to be checked. If a port is not specified, 
                the Real Server port is used. Specify 0 to unset CheckPort.
        type: int
    weight:
        description:
            -   When using weighted round robin scheduling, the weight of a
                Real Server is used to indicate what relative proportion of 
                traffic should be sent to the server. Servers with higher 
                values will receive more traffic.The weight of a SubVS can also
                be updated using the modrs command - set the rs to the number 
                that appears in the Id column for the relevant SubVS in 
                the parent Virtual Service modify screen. Values 1-65535.
        type: int
    check_host:
        description:
            -   The CheckUse1.1 parameter must be enabled to set the CheckHost 
                value. When using HTTP/1.1 checking, the Real Servers require 
                a Hostname be supplied in each request. 
                If no value is set then this value is the IP address 
                of the Virtual Service.
        type: str
    check_pattern:
        description:
            -   When the CheckType is set to http or https - this corresponds 
                to the Reply 200 Pattern in the WUI. This parameter only 
                applies when the HTTP Method is set to GET or POST. 
                When the CheckType is set to bdata: Specify the hexadecimal 
                string which will be searched for in the response. 
                Specify an empty value to unset CheckPattern.
        type: str
    check_headers:
        description:
            -   Specify up to four additional headers/fields 
                which will be sent with each health check request.
                Separate the pairs with a pipe, for example; 
                Host:xyc|UserAgent:prq.
        type: str
    check_use_11:
        description:
            -   By default, the health checker uses HTTP/1.0 when checking 
                the Real Server status. Enabling CheckUse1.1 means HTTP/1.1 is 
                used (which is more efficient).
        type: int
        choices:
            - 0
            - 1
    enhanced_health_checks:
        description:
            -   Enabling the EnhancedHealthchecks parameter provides an 
                additional health check parameter - RsMinimum. 
                If the EnhancedHealthchecks parameter is disabled, 
                the Virtual Service will be considered available 
                if at least one Real Server is available. 
                If the EnhancedHealthchecks parameter is enabled, 
                you can specify the minimum number of Real Servers 
                which should be available to consider the Virtual Service 
                to be available.
        type: int
        choices:
            - 0
            - 1
    rs_minimum:
        description:
            -   This parameter can only be set using the mod vs command if the 
                EnhancedHealthchecks parameter is enabled. Specify the minimum 
                number of Real Servers required to be available for the Virtual 
                Service to be considered up. If less than the minimum amount of 
                Real Servers are available, a critical log is generated. 
                If some Real Servers are down but it has not reached the 
                minimum amount specified, a warning is logged. If the email 
                options are configured, an email will be sent to the relevant 
                recipients. When retrieving the value of this parameter - 0 
                is the default value if there are no Real Servers or 1 Real 
                Server in the Virtual Service. However, 1 is always the 
                minimum in reality.
        type: int
    extra_header_key:
        description:
            -   Specify the key for the extra header to be inserted into 
                every request sent to the Real Servers.
        type: str
     extra_header_value:
        description:
            - 	Specify the value for the extra header to be inserted into 
                every request sent to the Real Servers.
        type: str
     error_code:
        description:
            -   If no Real Servers are available, the LoadMaster can 
                terminate the connection with a HTTP error code. Specify the 
                error code number in this parameter. To unset the error code, 
                set the parameter to an empty string.
        type: int
     error_url:
        description:
            -   When no Real Servers are available and an error response 
                is sent back to the client, 
                a redirect URL can also be specified.
        type: str
    ldap_endpoint:
        description:
            -   Specify the name of an LDAP endpoint to use for the health 
                checks. If LDAP is selected as the CheckType, the server IP 
                address (or addresses) and ports from the LDAP endpoint 
                configuration are used instead of the Real Server IP address 
                and port. For further information on LDAP endpoints
         type: str
    copy_header_from:
        description:
            -   This is the name of the source header field to copy into the 
                new header field before the request is sent to the 
                Real Servers.
        type: str
    copy_header_to:
        description:
            -   Used in conjunction with the CopyHdrFrom parameter. 
                The name of the header field into which the source header 
                is to be copied.
        type: str
    transparent:
        description:
            -   When using Layer 7, when this is enabled - the connection 
                arriving at the Real Server appears to come directly from 
                the client. Alternatively, the connection can be 
                non-transparent which means that the connections at the Real 
                Server appear to come from the LoadMaster. If a Virtual 
                Service (with or without a SubVS) has SSL re-encrypt enabled, 
                the transparency flag of the Virtual Service has no meaning 
                (re-encryption forces transparency to be off). 
                The transparency setting can still be modified by the API, 
                and is honored when re-encrypt is  disabled on the 
                Virtual Service.
        type: int
        choices:
            - 0
            - 1
    multi_connect:
        description:
            -   Enabling this option permits the LoadMaster to manage 
                connection handling between the LoadMaster and the Real 
                Servers. Requests from multiple clients will be sent over 
                the same TCP connection. Multiplexing only works for simple 
                HTTP GET operations. This parameter cannot be enabled 
                in certain situations, for example if WAF, ESP or SSL 
                Acceleration is enabled.
        type: int
        choices:
            - 0
            - 1
    non_local:
        description:
            -   By default only Real Servers on local networks can be assigned 
                to a Virtual Service. Enabling this option will allow a 
                non-local Real Server to be assigned to the Virtual Service.
                This option will only be available if non localrs 
                has been enabled and the Transparent option has been disabled 
                on the relevant Virtual Service.
        choices:
            - 0
            - 1
        type: int
    check_url:
        description:
            -   When the CheckType is set to http or https - by default, 
                the health checker tries to access the URL / to determine 
                if the machine is available. A different URL can be set in 
                the CheckUrl parameter. When the CheckType is set to bdata: 
                Specify a hexadecimal string to send to the Real Server.
                The maximum character length for the CheckUrl parameter 
                value is 126 characters.
        type: str
    check_post_data:
        description:
            -   This parameter is only relevant if the HTTP Method is 
                set to POST. When using the POST method, up to 2047 characters 
                of POST data can be sent to the server.
        type: str
    check_use_get:
        description:
            -   When accessing the health check URL - the system can 
                use the HEAD, the GET or the POST method
        type: int
        choices:
            - 0 
            - 1
            - 2
    persist:
        description:
            -   Specify the type of persistence (stickiness) to be used 
                for this Virtual Service.
        type: str
        choices:
            - ssl,
            - cookie,
            - active-cookie,
            - cookie-src,
            - cookie-hash,
            - cookie-hash-src,
            - url,
            - query-hash,
            - hash,
            - host,
            - header,
            - super,
            - super-src
            - src,
            - rdp,
            - rdp-src
            - rdp-sb,
            - rdp-sb-src,
            - udpsip,
            - none
    persist_timeout:
        description:
            -   The length of time (in seconds) after the last connection 
                that the LoadMaster will remember the persistence information. 
                Timeout values are rounded down to an even number of minutes. 
                Setting a value that is not a number of whole minutes results 
                in the excess being ignored. Setting a value to less than 60 
                seconds results in a value of 0 being set, 
                which disables persistency.
        type: int
    match_len:
        description:
            -   This parameter is only relevant when the CheckType is set to 
                bdata. Specify the number of bytes to find the CheckPattern 
                within. Values 0-8000.
        type: int
    stand_by_addr:
        description:
            -   Specify the IP address of the 'Sorry' server that is to be 
                used when no other Real Servers are available. This server will 
                not be health checked and is assumed to be always available.
        type: str
    stand_by_port:
        description:
            -   Specify the port of the 'Sorry' server.
        type: int
    schedule:
        description:
            -   Specify the type of scheduling of new connections to Real 
                Servers that is to be performed. 
        type: str
        choices:
            - Round-Robin
            - Weighted-Round-Robin
            - Least-Connection
            - Weighted-Least-Connection
            - Fixed-Weighting
            - Adaptive-Resource-Based
            - Source-IP-Hash
            - Weighted-Response-Time
            - SDN-Adaptive
            - URL-Hash
    rs_rule_precedence:
        description:
            -   This parameter should be used in conjunction with 
                RSRulePrecedencePos. This parameter is used to specify the 
                name of the existing rule whose position you wish to change.
        type: str
    rs_rule_precedence_pos:
        description:
            -   This parameter, in conjunction with the RSRulePrecedence 
                parameter, is used to change the position of the rule in a 
                sequence of rules. For example, a position of 2 means the rule 
                will be checked second.
        type: int
    selection_rules:
        description:
            -   add selection rules to a subvs, required a list of strings 
                of the rule names
        type: list (str)
    request_header_rules:
        description:
            -   add list of request rules to a subvs
        type: list (str)
    response_header_rules:
        description:
            -   add list of response rules to a subvs
        type: list (str)
"""

EXAMPLES = """
- name: Create a Sub VS
  hosts: localhost

  vars:
    central_address: '10.35.23.180'
    central_username: 'admin'
    central_api_key: '4ef39d1104767e18639bab'
    lm_address: '10.35.23.2:443'

  tasks:
  - name: Create SubVS
    sub_virtual_service:
        central_address: '{{ central_address }}'
        central_api_key: '{{ central_api_key }}'
        central_username: '{{ central_username }}'
        lm_address: '{{ lm_address }}'
        vs: '10.35.23.100'
        port: 80
        prot: 'tcp'
        nickname: 'Beta'
        vs_type: 'http'
        enable: 'Y'
        enhanced_health_checks: 1
        schedule: 'Round-Robin'
        content_rules: ['matchRedHeader']
"""

RETURN = '''
message:
    description: The message response indicating whether the task created or modified
    the Sub Virtual Service
    type: str
    returned: always
    sample: Sub VS Updated
changed:
    description: A boolean to indicate whether changes were made during the task
    type: bool
    returned: always
    sample: true
dataChanged:
    description: The parameters that were changed during the task
    type: str
    returned: when changed is true
    sample: {"Transparent": "N", "UseforSnat": "N", "VSPort": "0", "VStype": "http", "NickName": "Epsilon"}
msg:
    description: The error message related to why the task failed.
    type: str
    returned: when task failed
    sample:  The minimum supported LoadMaster firmware version is 7.2.47.0
'''

import requests
from ansible.module_utils.basic import AnsibleModule
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

API_MAP = {
    "AddVia": "add_via",
    "central_address": "central_address",
    "CheckCodes": "check_codes",
    "CheckHeaders": "check_headers",
    "CheckHost": "check_host",
    "CheckPattern": "check_pattern",
    "CheckPort": "check_port",
    "CheckPostData": "check_post_data",
    "CheckType": "check_type",
    "CheckUrl": "check_url",
    "CheckUse1.1": "check_use_11",
    "CheckUseGet": "check_use_get",
    "ContentRules": "content_rules",
    "CopyHdrFrom": "copy_header_from",
    "CopyHdrTo": "copy_header_to",
    "critical": "critical",
    "enable": "enable",
    "EnhancedHealthchecks": "enhanced_health_checks",
    "ErrorCode": "error_code",
    "ErrorPage": "error_page",
    "ExtraHdrKey": "extra_header_key",
    "ExtraHdrValue": "extra_header_value",
    "ErrorUrl": "error_url",
    "LdapEndpoint": "ldap_endpoint",
    "limit": "limit",
    "lm_address": "lm_address",
    "MatchLen": "match_len",
    "MultiConnect": "multi_connect",
    "NickName": "nickname",
    "non_local": "non_local",
    "Persist": "persist",
    "PersistTimeout": "persist_timeout",
    "port": "port",
    "prot": "prot",
    "QoS": "qos",
    "response_body_rules": "response_body_rules",
    "response_header_rules": "response_header_rules",
    "request_header_rules": "request_header_rules",
    "RsMinimum": "rs_minimum",
    "RSRulePrecedence": "rs_rule_precedence",
    "RSRulePrecedencePos": "rs_rule_precedence_pos",
    "Schedule": "schedule",
    "selection_rules": "selection_rules",
    "StandbyAddr": "stand_by_addr",
    "StandbyPort": "stand_by_port",
    "SubnetOriginating": "subnet_originating",
    "Transparent": "transparent",
    "weight": "weight",
    "vs": "vs",
    "VStype": "vs_type",
}


def run_module():
    module_args = dict(
        add_via=dict(type="int"),
        central_address=dict(required=True),
        central_api_key=dict(required=True),
        central_username=dict(required=True),
        check_codes=dict(type="int"),
        copy_header_from=dict(),
        copy_header_to=dict(),
        check_headers=dict(),
        check_host=dict(),
        check_pattern=dict(),
        check_port=dict(type="int"),
        check_post_data=dict(),
        check_type=dict(
            choices=[
                "icmp",
                "https",
                "http",
                "tcp",
                "smtp",
                "nntp",
                "ftp",
                "telnet",
                "pop3",
                "imap",
                "rdp",
                "bdata",
                "ldap",
                "none",
            ]
        ),
        check_url=dict(),
        check_use_11=dict(type="int", choices=[0,1]),
        check_use_get=dict(type="int", choices=[0,1,2]),
        content_rules=dict(type=list),
        critical=dict(type="int", choices=["0", "1"]),
        enable=dict(default="Y", choices=["Y", "N"]),
        enhanced_health_checks=dict(type="int", choices=[0,1]),
        ensure=dict(type="str", default="Present", choices=["Present", "Absent"]),
        error_code=dict(type="int"),
        error_url=dict(),
        extra_header_key=dict(),
        extra_header_value=dict(),
        limit=dict(type="int"),
        ldap_endpoint=dict(),
        lm_address=dict(required=True),
        match_len=dict(type="int"),
        multi_connect=dict(type="int", choices=[0, 1]),
        nickname=dict(required=True),
        non_local=dict(type="int", choices=[0, 1]),
        persist=dict(
            default=None,
            choices=[
                "ssl",
                "cookie",
                "active-cookie",
                "cookie-src",
                "cookie-hash",
                "cookie-hash-src",
                "url",
                "query-hash",
                "hash",
                "host",
                "header",
                "super",
                "super-src",
                "src",
                "rdp",
                "rdp-src",
                "rdp-sb",
                "rdp-sb-src",
                "udpsip",
                "none"
            ],
        ),
        persist_timeout=dict(type="int"),
        port=dict(type="int", required=True),
        prot=dict(type="str", choices=["tcp", "udp"]),
        qos=dict(),
        response_body_rules=dict(type="list", element="str"),
        response_header_rules=dict(type="list", element="str"),
        request_header_rules=dict(type="list", element="str"),
        rs_minimum=dict(type="int"),
        rs_rule_precedence=dict(type="str"),
        rs_rule_precedence_pos=dict(type="int"),
        schedule=dict(
            choices=[
                "Round-Robin",
                "Weighted-Round-Robin",
                "Least-Connection",
                "Weighted-Least-Connection",
                "Fixed-Weighting",
                "Adaptive-Resource-Based",
                "Source-IP-Hash",
                "Weighted-Response-Time",
                "SDN-Adaptive",
                "URL-Hash"
            ]
        ),
        selection_rules=dict(),
        stand_by_addr=dict(),
        stand_by_port=dict(type="int"),
        subnet_originating=dict(type="int"),
        transparent=dict(type="int", choices=[0,1]),
        vs=dict(required=True),
        vs_type=dict(
            required=True, choices=["gen", "http", "http2", "ts", "tls", "log"]
        ),
        weight=dict(int="int"),
    )

    module = AnsibleModule(
        argument_spec=module_args,
        mutually_exclusive=[
            ["error_code", "stand_by_addr"],
            ["non_local", "transparent"],
            ["error_url", "error_page"],
        ],
        required_together=[
            ["error_code", "error_url"],
            ["extra_header_key", "extra_header_value"],
            ["stand_by_addr", "stand_by_port"],
            ["copy_header_from", "copy_header_to"],
            ["rs_rule_precedence", "rs_rule_precedence_pos"],
        ],
        supports_check_mode=True,
    )
    result = dict(changed=False, original_message="", message="")

    result["message"] = module.params

    if module.check_mode:
        module.exit_json(**result)

    central_access_uri = "https://{}/api/v1/cms/subvs/".format(
        module.params.pop("central_address")
    )

    central_auth = {
        "username": module.params.pop("central_username"),
        "api_key": module.params.pop("central_api_key"),
    }

    response = requests.post(
        central_access_uri,
        params=central_auth,
        json=map_params_to_api(module.params),
        verify=False,
    )

    resp = response.json()

    if response.status_code is not 200:
        result = dict(changed=False)
        module.fail_json(msg=resp['message'], **result)
    
    if 'dataChanged' in resp:
        result = dict(changed=False, dataChanged='', message='')
        result['message'] = resp['message']
        result['dataChanged'] = resp['dataChanged']
        result['changed'] = resp['changed']
    else:
        result = dict(changed=False, message='')
        result['message'] = resp['message']
        result['changed'] = resp['changed']

    module.exit_json(**result)


def map_params_to_api(params):
    transformed = {}
    for api_key, input_key in API_MAP.items():
        if params.get(input_key, None):
            transformed[api_key] = params[input_key]
    return transformed


def main():
    run_module()


if __name__ == "__main__":
    main()
