#!/usr/bin/python

ANSIBLE_METADATA = {
    "metadata_version": "1.0",
    "status": ["preview"],
    "supported_by": "kemp",
}

DOCUMENTATION = """
---
module: virtual_service

short_description: Module to add or modify a Virtual Service on a LoadMaster.

version_added: "1.0"

description:
    - "Module to add or modify a Virtual Services on a LoadMaster. The minumum 
    supported LoadMaster firmware virsion is 7.2.47.0."

options:
    allow_https_2:
        description:
            -   Enable HTTP2 for this Virtual Service. SSL Acceleration must be
            enabled before HTTP2 can be enabled. The Best Practices cipher set
            should be used when HTTP2 is enabled.
        type: str
        choices:
            Y: Enabled
            N: Disabled
    central_address:
        description:
            -   The Central IP address where the LoadMaster is added to.
        required: true
        type: str
    central_api_key:
        description:
            -   Admin-level API Key to access API services on Kemp360Central.
        type: str
        required: true
    central_username:
        description:
            -   Username for Kemp360Central that is linked to the given API key
        type: str
        required: true
    cert_name:
        description:
            - Identifier (name) of a preexisting certificate on the LoadMaster
            to assign to the Virtual Service.
        type: str
        required: false
    check_host:
        description:
            -  	The CheckUse1.1 parameter must be enabled to set the CheckHost value.
            When using HTTP/1.1 checking, the Real Servers require a Hostname be supplied 
            in each request. If no value is set then this value is the IP address
            of the Virtual Service.
        type: str
        required: false
    check_pattern:
        description:
            -  When the CheckType is set to http or https
            this corresponds to the Reply 200 Pattern in the WUI.
            This parameter only applies when the HTTP Method is set to
            GET or POST. 
            When the CheckType is set to bdata: Specify the hexadecimal
            string which will be searched for in the response. Specify
            an empty value to unset CheckPattern.
        required: false
        type: str
    check_port:
        description:
            -   The port to be checked. If a port is not specified, 
                the Real Server port is used. Specify 0 to unset CheckPort.
        type: int
    check_post_data:
        description:
            -   This parameter is only relevant if the HTTP Method is set to
            POST. When using the POST method, up to 2047 characters of POST
            data can be sent to the server.
        type: str
        required: false
    check_type:
        description:
            - Specify which protocol is to be used to check the health
            of the Real Server. The default value is dependent on the
            Virtual Service port.
        type: str
        required: false
        choices:
            - icmp
            - https
            - http
            - tcp
            - smtp
            - nntp
             -ftp
            - telnet
            - pop3
            - imap
            - rdp
            - bdata
            - ldap
            - none 
    check_url:
        description:
            -   When the CheckType is set to http or https - by default, 
                the health checker tries to access the URL / to determine 
                if the machine is available. A different URL can be set in 
                the CheckUrl parameter. When the CheckType is set to bdata: 
                Specify a hexadecimal string to send to the Real Server.
                The maximum character length for the CheckUrl parameter 
                value is 126 characters.
        type: str
    check_use_11:
        description:
            -   By default, the health checker uses HTTP/1.0 when checking 
                the Real Server status. Enabling CheckUse1.1 means HTTP/1.1 is 
                used (which is more efficient).
        type: str
        choices:
            - N: Disabled
            - Y: Enabled
    check_use_get:
        description:
            -   When accessing the health check URL - the system can 
                use the HEAD, the GET or the POST method
        type: int
        choices:
            - 0: HEAD
            - 1: GET
            - 2: POST
    cipher_set:
        description:
            -   This parameter can be used to assign a cipher set to
            a Virtual Service. System-defined cipher sets and custom
            cipher sets can be assigned using this parameter.
        type: str
        required: false
        choices:
            - Default
            - Default_NoRc4
            - BestPractices
            - Intermediate_compatibility
            - Backward_compatibility
            - WUI
            - FIPS
            - Legacy
            - Null_Ciphers
            - <NameOfCustomCipherSet>
    ciphers:
        description:
            - Multiple ciphers can be assigned by inserting a colon between 
            each cipher. When ciphers are assigned in this way, a Cipher Set 
            called Custom_<VirtualServiceID> will be created/updated. Note: The 
            assigned ciphers list will be overwritten when ciphers are added in 
            this way. Ensure to include all ciphers to be assigned.
        type: str
    enable:
        description:
            -   Specify if the Virtual Service should be created in a live
            (enabled) state.
        type: str
        choices:
            - N: Disabled
            - Y: Enabled
        required: True
        default: Y
    enhanced_health_checks:
        description:
            -   Enabling the EnhancedHealthchecks parameter provides an 
                additional health check parameter - RsMinimum. 
                If the EnhancedHealthchecks parameter is disabled, 
                the Virtual Service will be considered available 
                if at least one Real Server is available. 
                If the EnhancedHealthchecks parameter is enabled, 
                you can specify the minimum number of Real Servers 
                which should be available to consider the Virtual Service 
                to be available.
        type: int
        choices:
            - 0: Disabled
            - 1: Enabled
    ensure:
        description:
            - Value set to indicate to Central that this Virtual Service should
            always exist. This is set automatically by the module.
        required: True
        default: Present
        type: str
    force_l7:
        description:
            - Enabling ForceL7 means the Virtual Service runs at Layer 7
             and not Layer 4. This may be needed for various reasons,
             including that only Layer 7 services can be non-transparent.
        type: int
        required: false
        choices:
            - 0: Disabled
            - 1: Enabled
    ip:
        description:
            - The IPv4 Address to assign to the Virtual Service.
        type: str
        required: true
    ldap_endpoint
        description:
            -   Specify the name of an LDAP endpoint to use for the
            health checks. If LDAP is selected as the CheckType,
            the server IP address (or addresses) and ports from 
            the LDAP endpoint configuration are used instead of
            the Real Server IP address and port.
        required: false
        type: str
    lm_address:
        description:
            -   IP address and port of the LoadMaster that contains the Virtual
                Service or Sub Virtual Service that the Real Server should be
                created on modified on. The format is 'ip:port'.
        required: true
        type: str
    match_body_rules
        description:
            - Names (Identifiers) of Match Body type Content Rules to assign to
            the Virtual Service. These content rules must exist on the LoadMaster
            prior to being assigned to a Virtual Service.
        type: list
        required: false
    match_length:
        description:
            -   This parameter is only relevant when the CheckType is 
            set to bdata. Settign this you can specify the number of bytes
            to find the CheckPattern within.
        type: int
        required: false
    need_host_name:
        description:
            - When this parameter is enabled, the hostname is always 
            required to be sent in the TLS client hello message.
            If it is not sent, the connection will be dropped.
        type: int
        required: false
        choices:
            - 0: Disabled
            - 1: Enabled
    nickname:
        description:
            - Nickname to assign to the Virtual Service. Must be unique.
        required: true
        type: str
    ocsp_verify:
        description:
            - Verify (using Online Certificate Status Protocol (OCSP))
            that the client certificate is valid.
        type: int
        choices:
            - 0: Disabled
            - 1: Enabled
    persist:
        description:
            -   Specify the type of persistence (stickiness) to be used 
                for this Virtual Service.
        type: str
        choices:
            - ssl,
            - cookie,
            - active-cookie,
            - cookie-src,
            - cookie-hash,
            - cookie-hash-src,
            - url,
            - query-hash,
            - hash,
            - host,
            - header,
            - super,
            - super-src
            - src,
            - rdp,
            - rdp-src
            - rdp-sb,
            - rdp-sb-src,
            - udpsip,
            - none
    persist_timeout:
        description:
            -   The length of time (in seconds) after the last connection 
                that the LoadMaster will remember the persistence information. 
                Timeout values are rounded down to an even number of minutes. 
                Setting a value that is not a number of whole minutes results 
                in the excess being ignored. Setting a value to less than 60 
                seconds results in a value of 0 being set, 
                which disables persistency.
        type: int
    port:
        description:
            -   The port on which the Virtual Service must be active. Can be
            any valid port number from 3-65530, or a wildcard `*`
        type: int
        required: true
    preprocess_rules:
         description:
            - Names (Identifiers) of Preprocess type Content Rules to assign to
            the Virtual Service. These content rules must exist on the LoadMaster
            prior to being assigned to a Virtual Service.
        type: list
        required: false
    protocol:
        description:
            -   The protocol type that this Virtual Service will use.
        type: str
        required: true
        choices:
            - tcp: Use the TCP protocol
            - udp: Use the UDP protocol
    qos:
        description:
            -   Quality of Service sets a type of service that deals with 
                packets will treat and prioritise the traffic
        required: false
        type: str
        choices:
            - Normal-Service
            - Minimize-Cost
            - Maximize-Reliability
            - Maximize-Throughput
            - Minimize-Delay
    request_rules:
         description:
            - Names (Identifiers) of Request type Content Rules to assign to
            the Virtual Service. These content rules must exist on the LoadMaster
            prior to being assigned to a Virtual Service.
        type: list
        required: false
    response_rules:
         description:
            - Names (Identifiers) of Response type Content Rules to assign to
            the Virtual Service. These content rules must exist on the LoadMaster
            prior to being assigned to a Virtual Service.
        type: list
        required: false
    rs_minimum:
        description:
            -   Can only be set if enhanced_health_checks is set to 'Y'.
            Specify the minimum number of Real Servers required to be
            available for the Virtual Service to be considered up
        type: int
        required: false
    rs_rule_precedence:
        description:
            -   This parameter should be used in conjunction with 
            RSRulePrecedencePos. This parameter is used to specify the
            name of the existing rule whose position you wish to change.
        required: false
        type: str
    rs_rule_precedence_pos:
        description:
            -   This parameter, in conjunction with the
            RSRulePrecedence parameter, is used to change the position of
            the rule in a sequence of rules. For example, a position of 2
            means the rule will be checked second.
        type: int
    schedule:
        description:
            -   Specify the type of scheduling of new connections to Real 
                Servers that is to be performed. 
        type: str
        choices:
            - Round-Robin
            - Weighted-Round-Robin
            - Least-Connection
            - Weighted-Least-Connection
            - Fixed-Weighting
            - Adaptive-Resource-Based
            - Source-IP-Hash
            - Weighted-Response-Time
            - SDN-Adaptive
            - URL-Hash
    ssl_acceleration:
        description:
            -   Enable SSL handling services for the Virtual Service.
        type: int
        required: false
        choices:
            - 0: Disabled
            - 1: Enabled
    ssl_reencrypt:
        description:
            -   When this option is enabled, the SSL data stream is 
            re-encrypted before sending to the Real Server. This parameter is
            only valid if SSL Acceleration is enabled.
        type: int
        required: false
        choices:
            - 0: Disabled
            - 1: Enabled

    ssl_rewrite:
        description:
            -   When the Real Server rejects a request with a HTTP redirect,
            the requesting Location URL may need to be converted to specify 
            HTTPS instead of HTTP (and vice versa).
        type: str
        required: false
        choices:
            - None
            - http
            - https
    tls_type:
        description:
            -    Specify which of the following protocols to support; 
            SSLv3, TLS1.0, TLS1.1, TLS1.2, or TLS1.3.
        type: list
        required: false
        choices:
            - SSLv3
            - TLS1.0
            - TLS1.1
            - TLS1.2
            - TLS1.3
    transparent:
        description:
            -   (Layer 7 Only) When transparency is enabled connections
            at the Real Server will appear to originate at the client. With
            transparency disable, connections will originate at the LoadMaster.
        type: int
        required: false
        choices:
            - 1: Enabled
            - 0: Disabled
    use_for_snat:
        description:
            -   By default, when the LoadMaster is being used to 
            NAT Real Servers, the source IP address used on the Internet is
            that of the LoadMaster. Enabling this option allows the Real 
            Servers configured to use the Virtual Service as the source IP
            address instead. If the Real Servers are configured on more than
            one Virtual Service which has this option set, only connections
            to destination port 80 will use this Virtual Service as the source
            IP address.
        type: int
        required: false
        choices:
            - 1: Enabled
            - 0: Disabled
    vs_type:
        description:
            -   Specifies the type of service being load balanced.
        type: str
        required: true
        choices:
            - gen
            - http
            - http2
            - log
            - ts
            - tls

"""

EXAMPLES = """
- name: Create a Virtual Service
  hosts: localhost

  vars:
    central_address: '10.35.23.180'
    central_username: 'admin'
    central_api_key: '4ef39d110474a18639bab'
    lm_address: '10.35.23.2:443'
    ip: '10.35.23.156'
    port: 443
    prot: 'tcp'

  tasks:
    - name: Create Virtual Service Pathos on LM
      virtual_service:
        central_address: '{{ central_address }}'
        central_username: '{{ central_username }}'
        central_api_key: '{{ central_api_key }}'
        lm_address: '{{ lm_address }}'
        enable: 'Y'
        nickname: 'Pathos'
        ip: '{{ ip }}'
        port: '{{ port }}'
        protocol: '{{ prot }}'
        vs_type: 'http'
        ssl_acceleration: 1
        check_type: 'icmp'
        qos: 'Maximize-Reliability'
        transparent: 1
"""

RETURN = '''
message:
    description: The message response indicating whether the task created or modified
    the Virtual Service
    type: str
    returned: always
    sample: VS Updated
changed:
    description: A boolean to indicate whether changes were made during the task
    type: bool
    returned: always
    sample: true
dataChanged:
    description: The parameters that were changed during the task
    type: str
    returned: when changed is true
    sample: {"CheckType": "icmp","NickName": "Pathos","SSLAcceleration": "Y", "TlsType": "3", "Transparent": "Y"}
msg:
    description: The error message related to why the task failed.
    type: str
    returned: when task failed
    sample:  The minimum supported LoadMaster firmware version is 7.2.47.0
'''

import json
import requests

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils import utils
import urllib3

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

API_MAP = {
    "AllowHTTP2": "allow_http_2",
    "central_address": "central_address",
    "CertFile": "cert_name",
    "CheckHost": "check_host",  # Can only set if checkuse_11 is set.
    "CheckPattern": "check_pattern",
    "CheckPort": "check_port",
    "CheckPostData": "check_post_data",
    "CheckType": "check_type",
    "CheckUrl": "check_url",
    "CheckUse1.1": "check_use_11",
    "CheckUseGet": "check_use_get",
    "Ciphers": "ciphers",
    "CipherSet": "cipher_set",
    "Enable": "enable",
    "EnhancedHealthchecks": "enhanced_health_checks",
    "ForceL7": "force_l7",
    "ip": "ip",
    "LdapEndpoint": "ldap_endpoint",
    "lm_address": "lm_address",
    "MatchBodyRules": "match_body_rules",
    "MatchLen": "match_length",
    "NeedHostName": "need_host_name",
    "NickName": "nickname",
    "OCSPVerify": "ocsp_verify",
    "Persist": "persist",
    "PersistTimeout": "persist_timeout",
    "port": "port",
    "PreprocessRules": "preprocess_rules",
    "protocol": "protocol",
    "QoS": "qos",
    "RequestRules": "request_rules",
    "ResponseRules": "response_rules",
    "RSMinimum": "rs_minimum",
    "RSRulePrecedence": "rs_rule_precedence",
    "RSRulePrecedencePos": "rs_rule_precedence_pos",
    "Schedule": "schedule",
    "SSLAcceleration": "ssl_acceleration",
    "SSLReencrypt": "ssl_reencrypt",
    "SSLreverse": "ssl_reverse",
    "SSLRewrite": "ssl_rewrite",
    "TLSType": "tls_type",
    "Transparent": "transparent",
    "UseForSnat": "use_for_snat",
    "VStype": "vs_type"
}


def run_module():
    module_args = dict(
        allow_http_2=dict(type="str", choices=["Y", "N"]),
        central_address=dict(type="str", required=True),
        central_api_key=dict(type="str", required=True),
        central_username=dict(type="str", required=True),
        cert_name=dict(type="str"),
        check_host=dict(type="str"),
        check_pattern=dict(type="str"),
        check_port=dict(type="int"),
        check_post_data=dict(type="str"),
        check_type=dict(
            type="str",
            choices=[
                "icmp",
                "https",
                "http",
                "tcp",
                "smtp",
                "nntp",
                "ftp",
                "telnet",
                "pop3",
                "imap",
                "rdp",
                "bdata",
                "ldap",
                "none",
            ],
        ),
        check_url=dict(type="str"),
        check_use_11=dict(type="str", choices=["Y", "N"]),
        check_use_get=dict(type="int"),
        cipher_set=dict(type="str"),
        ciphers=dict(type="str"),
        enable=dict(type="str", default="Y", choices=["Y", "N"]),
        enhanced_health_checks=dict(type="int"),
        ensure=dict(type="str", default="Present", choices=["Present", "Absent"]),
        force_l7=dict(type="int"),
        ip=dict(type="str", required=True),
        ldap_endpoint=dict(type=str),
        lm_address=dict(type="str", required=True),
        match_length=dict(type="int"),
        match_body_rules=dict(type="list"),
        need_host_name=dict(type="int"),
        nickname=dict(type="str"),
        ocsp_verify=dict(type="int"),
        persist=dict(
            type="str",
            default=None,
            choices=[
                "ssl",
                "cookie",
                "active-cookie",
                "cookie-src",
                "cookie-hash",
                "cookie-hash-src",
                "url",
                "query-hash",
                "hash",
                "host",
                "header",
                "super",
                "super-src"
                "src",
                "rdp",
                "rdp-src"
                "rdp-sb",
                "rdp-sb-src",
                "udpsip",
                "none"
            ],
        ),
        persist_timeout=dict(type="int"),  # no default = kemptech api breaks
        port=dict(type="int", required=True),
        preprocess_rules=dict(type="list"),
        protocol=dict(type="str", required=True, choices=["udp", "tcp"]),
        qos=dict(type="str", choices=[
            "Normal-Service",
            "Minimize-Cost",
            "Maximize-Reliability",
            "Maximize-Throughput",
            "Minimize-Delay"
        ]),
        request_rules=dict(type="list"),
        response_rules=dict(type="list"),
        rs_minimum=dict(type="int"),
        rs_rule_precedence=dict(type="str"),
        rs_rule_precedence_pos=dict(type="int"),
        schedule=dict(
            type="str",
            choices=[
                "Round-Robin",
                "Weighted-Round-Robin",
                "Least-Connection",
                "Weighted-Least-Connection",
                "Fixed-Weighting",
                "Adaptive-Resource-Based",
                "Source-IP-Hash",
                "Weighted-Response-Time",
                "SDN-Adaptive",
                "URL-Hash",
            ],
        ),
        ssl_acceleration=dict(type="int"),
        ssl_reencrypt=dict(type="int"),
        ssl_reverse=dict(type="int"),
        ssl_rewrite=dict(type="str", choices=["none", "http", "https"]),
        tls_type=dict(type="int"),
        transparent=dict(type="int"),
        use_for_snat=dict(type="int"),
        vs_type=dict(
            type="str",
            required=True,
            choices=["gen", "http", "http2", "ts", "tls", "log"],
        ),
    )

    module = AnsibleModule(argument_spec=module_args, supports_check_mode=True)
    central_auth = utils.get_central_auth(module.params)
    virtual_service = utils.map_params_to_api(module.params, API_MAP)
    result = dict(changed=True, original_message="", message="")

    if module.check_mode:
        module.exit_json(**result)

    central_access_uri = "https://{}/api/v1/cms/vs/".format(
        virtual_service["central_address"]
    )
    result["original_message"] = virtual_service
    response = requests.post(
        central_access_uri, params=central_auth, json=virtual_service, verify=False
    )
    resp = response.json()

    if response.status_code is not 200:
        result = dict(changed=False)
        module.fail_json(msg=resp, **result)

    if "dataChanged" in resp:
        result = dict(changed=False, dataChanged="", message="")
        result["message"] = resp["message"]
        result["dataChanged"] = resp["dataChanged"]
        result["changed"] = resp["changed"]
    else:
        result = dict(changed=False, message="")
        result["message"] = resp["message"]
        result["changed"] = resp["changed"]
    module.exit_json(**result)


def main():
    run_module()


if __name__ == "__main__":
    main()
